function read_isf
% FUNCTION READ_ISF
% Read ISF formatted data from a ASCII file in the format of ISF1.0. 
% 
% Input data is read in from a specfied .isf file containing the ISF data.
% This function produces a matlab table, containign the ISF-formatted data.
% From the input ISF file, a .mat file is produced with the same file name.
% 
% Tom Kettlety, University of Oxford, 2025
% tom.kettlety@earth.ox.ac.uk
% 


fname = 'EXAMPLE_ISF_BULLETIN';


% open file
fid = fopen([floc,fname,'.isf'],'r');
% nl = linecount(fid);

% start event counter
n = 0;

% preset event catalogue structure
c.N = NaN;

% get first line
tline = fgetl(fid);

while ischar(tline)
    %     tmpstr = strsplit(tline,' ');
    
    % check if line has got anything in it
    if ~isempty(tline) && ~strcmp(tline,' ')
        
        % if current line indicates a new event, look for phase readings
        if strcmp(tline(1:5),'Event')
            
            % count events found
            n = n + 1;

            % get listed event number from line
            nev = str2double(tline(6:14));
            
            % print progress to command window
            fprintf('Event %d found, called %d: ',n,nev)
            
%             % checking if there are any issues with event numbers not
%             % matching
%             ntest = str2double(tline(6:end));
%             if ntest~=n
%                 debug = 1;
% %                 warning(['Event number (%d) is not the same as file ',...
% %                     'event number (%d)'],n,ntest)
%             end
            
            % skip over header line to get to data
            tline = fgetl(fid);
            
            % read the origin table and save to c structure
            c = read_origin_table(fid,c,n,nev);
            
            fprintf('%s, ',datestr(c(n).T(1),'yyyy/mm/dd HH:MM:SS.FFF'));
            
            
            % go into new search through file
            while ischar(tline)
                
                % check if line has got anything in it
                if ~isempty(tline) && ~strcmp(tline,' ')
                    
                    % if it's the start of the phase table, start recording
                    if strcmp(tline(1:9),'Magnitude')
                        
                        % read the magnitude table and save to c structure
                        c = read_magnitude_table(fid,c,n);
                        
                        try
                            fprintf('mags found (Mag: %.2f), ',...
                                c(n).Magnitude(1))
                        catch
                            fprintf('just mag headers, ')
                        end
                        
%                     elseif strcmp(tline(1:7),'Effects')
%
%                         % read the phase table and save to ph structure
%                         c = read_effects_table(fid,c,n);
                        
                    elseif strcmp(tline(1:3),'Sta')
                        
                        % read the phase table and save to ph structure
                        c = read_phase_table(fid,c,n);
                        
                        try
                            fprintf('phases found (NPh: %d), ',length(c(n).Sta))
                        catch
                            fprintf('just phase headers, ')
                        end
                        
                        % see if it's a new event
                    elseif strcmp(tline(1:5),'Event')
                        
                        fprintf('done.\n')
                        
                        % skip back to previous line and go back to primary
                        % while loop
                        fseek(fid,fpos,-1);
                        break
                    elseif strcmp(tline(1:4),'STOP')
                        break
                    end
                    
                end
                
                % get position of this line, so that it can be traced back to if
                % next is a new event
                fpos = ftell(fid);
                
                % get next line
                tline = fgetl(fid);
            end
            
        end
    end
    
    tline = fgetl(fid);
end

fprintf('\n\nReading file %s complete.\n\n',[floc,fname,'.isf'])

% close file
fclose(fid);

% convert to table for easy reading/analysis
c = struct2table(c);

% extract prime times from bulletin (first in row)
ne = height(c);
T = nan(ne,1);
for i = 1:ne
    T(i) = c.T{i}(1);
end

% use prime times to sort events acending in time
[~,idx] = sort(T,'ascend');
c = c(idx,:);

% save to file
bSave = true;
if bSave
    save([floc,fname,'_isf.mat'],'c');
end




end


function n = linecount(fid)
n = 0;
tline = fgetl(fid);
while ischar(tline)
    tline = fgetl(fid);
    n = n+1;
end
frewind(fid);
end




function c = read_origin_table(fid,c,n,nev)

% if this is the first time doing this, set up the number of events tracker
if length(c)==1 && n==1
    c(n).N = nev;
else
    if n~=length(c)+1
        debug = 1;
    else
        c(n).N = nev;
    end
end

% ??? why not just:
% c(n).N = n;
% 
% % set number of events read in so far
% n = length(c);

% get first line
tline = fgetl(fid);

% initialise line tracker
l = 0;

% loop through data theres stuff in the line, not just a space say
while ~isempty(tline) && ischar(tline) && ~strcmp(tline,' ')
    % check length of line
    if ~isnan(str2double(tline(1:4))) % length(tline)>=136
        % increment line counter of table
        l = l+1;
        
        % read in orgin block data from specific character locations
        c(n).Date(l,1) = {strtrim(tline(1:10))};
        c(n).Time(l,1) = {strtrim(tline(12:22))};
        c(n).TFixed = {strtrim(tline(23))};
        c(n).TErr(l,1) = str2double(tline(25:29));
        c(n).RMS(l,1) = str2double(tline(31:35));
        c(n).Latitude(l,1) = str2double(tline(37:44));
        c(n).Longitude(l,1) = str2double(tline(46:54));
        c(n).EpiFixed(l,1) = {strtrim(tline(55))};
        c(n).Smaj(l,1) = str2double(tline(56:60));
        c(n).Smin(l,1) = str2double(tline(62:66));
        c(n).Az(l,1) = str2double(tline(68:70));
        c(n).Depth(l,1) = str2double(tline(72:76));
        c(n).ZFixed(l,1) = {strtrim(tline(77))};
        c(n).ZErr(l,1) = str2double(tline(79:82));
        c(n).Ndef(l,1) = str2double(tline(84:87));
        c(n).Nsta(l,1) = str2double(tline(89:92));
        c(n).Gap(l,1) = str2double(tline(94:96));
        c(n).mdist(l,1) = str2double(tline(98:103));
        c(n).Mdist(l,1) = str2double(tline(105:110));
        c(n).AType(l,1) = {strtrim(tline(112))};
        c(n).LocMeth(l,1) = {strtrim(tline(114))};
        c(n).EvType(l,1) = {strtrim(tline(116:117))};
        c(n).Author(l,1) = {strtrim(tline(119:127))};
        c(n).OrigID(l,1) = {strtrim(tline(129:end))};
        c(n).Comment(l,1) = {[]};
        
        % add on extra columns with time in matlab format
        if contains(tline(1:22),'.')
            c(n).T(l,1) = datenum(tline(1:22),'yyyy/mm/dd HH:MM:SS.FFF');
        else
            c(n).T(l,1) = datenum(tline(1:22),'yyyy/mm/dd HH:MM:SS');
        end
        c(n).Tdt(l,1) = datetime(c(n).T(l,1),'ConvertFrom','datenum');
    else
        % 
        if ~contains(tline,'#') && ~contains(tline,'+')
            debug = 1;
        end
        
        try
            if isempty(c(n).Comment(l,1))
                c(n).Comment(l,1) = {strtrim(tline)};
            else
                c(n).Comment(l,1) = {[c(n).Comment{l,1},strtrim(tline)]};
            end
        catch
            c(n).Comment(l,1) = {strtrim(tline)};
        end
    end
    
    tline = fgetl(fid);
end



end


function c = read_magnitude_table(fid,c,n)

tline = fgetl(fid);
l = 0;
while ~isempty(tline) && ischar(tline) && ~strcmp(tline,' ')
    if length(tline)>=21
        l = l+1;
        c(n).MagType(l,1) = {strtrim(tline(1:5))};
        c(n).MinMaxInd(l,1) = {strtrim(tline(6))};
        c(n).Magnitude(l,1) = str2double(tline(7:10));
        c(n).MagErr(l,1) = str2double(tline(12:14));
        c(n).MagNsta(l,1) = str2double(tline(16:19));
        if length(tline)<29
            c(n).MagAuthor(l,1) = {strtrim(tline(21:end))};
            c(n).MagOrigID(l,1) = {''};
        else
            c(n).MagAuthor(l,1) = {strtrim(tline(21:29))};
            if length(tline)>30
                c(n).MagOrigID(l,1) = {strtrim(tline(31:end))};
            end
        end
        
        
        c(n).MagComment(l,1) = {''};
        
        
    else
        if ~contains(tline,'#') 
            debug = 1;
        end
        
        try
            if isempty(c(n).MagComment(l,1))
                c(n).MagComment(l,1) = {strtrim(tline)};
            else
                c(n).MagComment(l,1) = {[c(n).MagComment{l,1},strtrim(tline)]};
            end
        catch
            c(n).MagComment(l,1) = {strtrim(tline)};
        end
    end
    
    tline = fgetl(fid);
end

end


function c = read_phase_table(fid,c,n)

tline = fgetl(fid);
l = 0;
linetracker = '';
while ~isempty(tline) && ischar(tline) && ~strcmp(tline,' ')
    
    % don't include complete repeated lines
    if ~contains(linetracker,tline)
        
        % make sure theres stuff actually in the line, and that it's the
        % expected length to read in
        if length(tline)>=102
            % keep track of the number of rows in the table
            l = l+1;
            
            % read in the parameters from the table
            c(n).Sta(l,1) = {strtrim(tline(1:5))};
            c(n).Dist(l,1) = str2double(tline(7:12));
            c(n).EvAz(l,1) = str2double(tline(14:18));
            c(n).Phase(l,1) = {strtrim(tline(20:27))};
            c(n).PhTime(l,1) = {strtrim(tline(29:40))};
            c(n).TRes(l,1) = str2double(tline(42:46));
            c(n).Azim(l,1) = str2double(tline(48:52));
            c(n).AzRes(l,1) = str2double(tline(54:58));
            c(n).Slow(l,1) = str2double(tline(60:65));
            c(n).SRes(l,1) = str2double(tline(67:72));
            c(n).Def(l,1) = {strtrim(tline(74:76))};
            c(n).SNR(l,1) = str2double(tline(78:82));
            c(n).Amp(l,1) = str2double(tline(84:92));
            c(n).Per(l,1) = str2double(tline(94:98));
            c(n).PickType(l,1) = {strtrim(tline(100))};
            c(n).MotionDir(l,1) = {strtrim(tline(101))};
            c(n).OnsetQual(l,1) = {strtrim(tline(102))};
            
            % had to impliment for BGR, as formatting is bad. need to check
            % if the table has the last couple of columns. if not, make
            % them empty
            if length(tline)>102
                c(n).PhMagType(l,1) = {strtrim(tline(104:108))};
                c(n).PhMagMinMax(l,1) = {strtrim(tline(109))};
                c(n).PhMagnitude(l,1) = str2double(tline(110:113));
                c(n).ArrID(l,1) = {strtrim(tline(115:end))};
            else
                c(n).PhMagType(l,1) = {''};
                c(n).PhMagMinMax(l,1) = {''};
                c(n).PhMagnitude(l,1) = NaN;
                c(n).ArrID(l,1) = {''};
            end
            
            c(n).PhComment(l,1) = {[]};
            
            % keep track of what's in the phase block to check for totally 
            % repeated lines, ignoring commments
            linetracker = [linetracker tline];
        else
            %  this else expects this line to be comments. usually these
            %  have a # in them. if not, can put a breakpoint on the below
            %  to debug.
            if ~contains(tline,'#')
                debug = 1;
            end
            
            try
                if isempty(c(n).PhComment(l,1))
                    c(n).PhComment(l,1) = {strtrim(tline)};
                else
                    c(n).PhComment(l,1) = {[c(n).PhComment{l,1},...
                        strtrim(tline)]};
                end
            catch
                c(n).PhComment(l,1) = {strtrim(tline)};
            end
        end
    end
    
    
    tline = fgetl(fid);
    if ~isempty(tline) && ~strcmp(tline,' ') && ~isa(tline,'numeric')
        if strcmp(tline(1:3),'Net')
            debug = 1;
        end
    end
end

end










