function write_isf
% FUNCTION WRITE_ISF
% Write ISF formatted data to a ASCII file in the format of ISF1.0. 
% 
% Input data is read in from a specfied .mat file containing the ISF data table.
% This matlab table could be generated manually, or made from an existing ISF 
% file using the function contained in "read_isf.m". A corresponding text file 
% is produced with the same file name, but a ".isf" file extension.
% 
% Tom Kettlety, University of Oxford, 2025
% tom.kettlety@earth.ox.ac.uk
% 



fname = 'EXAMPLE_ISF_BULLETIN.mat';

fprintf('Loading %s... ',fname)
tmp = load(fname);
c = tmp.c;
fprintf('done. \n')
ne = height(c);

% event to start writing to file
nstart = 1;

% event to stop writing
nend = ne;


%% write to ISF
fid = fopen([fname(1:end-4),'.isf'],'w');

% bulletin title block
fprintf(fid,'DATA_TYPE BULLETIN IMS1.0');
fprintf(fid,'\n');
fprintf(fid,'(# Write time: %s)',datestr(now,'yyyy-mm-ddTHH:MM:SS'));
fprintf(fid,'\n\n');

nent = 0;
for i = nstart:nend
    % print status to command line 
    fprintf('%s: Writing event %d of %d... ',datestr(now,'yyyy-mm-ddTHH:MM:SS'),i,ne)

    % event title block
    fprintf(fid,'Event ');
    fprintf(fid,'%8d ',i);
    fprintf(fid,'North Sea');
    fprintf(fid,'\n');
    
    %% write out origin block
    % header
    fprintf(fid,['   Date       Time        Err   RMS Latitude ',...
        'Longitude  Smaj  Smin  Az Depth   Err Ndef Nsta Gap  mdist  ',...
        'Mdist Qual   Author      OrigID']);
    fprintf(fid,'\n');
    
    % setting up variables to keep track of what OrigIDs have been used
    % below, in order to use them again in the magnitude block.
    OIDtrack = nan(size(c.T{i}));
    OIDassign = nan(size(c.T{i}));
    
    % loop through each entry in the block
    for j = 1:length(c.T{i})
        % keep track on entry number through whole file. needed to match
        % up correct OrigIDs
        nent = nent + 1;
        
        % write data with specific ISF character locations
        fprintf(fid,'%s ',c.Date{i}{j});
        fprintf(fid,'%s',c.Time{i}{j}); 
        fprintf(fid,'  ');
        printwithcheck(fid,c.TErr{i}(j),'%5.2f ',6)
        printwithcheck(fid,c.RMS{i}(j),'%5.2f ',6)
        printwithcheck(fid,c.Latitude{i}(j),'%8.4f ',9)
        printwithcheck(fid,c.Longitude{i}(j),'%9.4f',9)
        printwithcheck(fid,c.EpiFixed{i}{j},'%s', 1)
        printwithcheck(fid,c.Smaj{i}(j),'%5.1f ',6)
        printwithcheck(fid,c.Smin{i}(j),'%5.1f ',6)
        printwithcheck(fid,c.Az{i}(j),'%3.0f ',4)
        printwithcheck(fid,c.Depth{i}(j),'%5.1f',5)
        printwithcheck(fid,c.ZFixed{i}{j},'%s ',2)
        printwithcheck(fid,c.ZErr{i}(j),'%4.1f ',5)
%         if isnan(c.Ndef{i}(j)) && length(c.PhTime{i})>=1 && j==1
%             c.Ndef{i}(j) = length(c.PhTime{i});
%         end
        printwithcheck(fid,c.Ndef{i}(j),'%4.0f ',5)
%         if isnan(c.Nsta{i}(j)) && length(unique(c.Sta{i}))>=1 && j==1
%             c.Nsta{i}(j) = length(unique(c.Sta{i}));
%         end
        printwithcheck(fid,c.Nsta{i}(j),'%4.0f ',5)
        printwithcheck(fid,c.Gap{i}(j),'%3.0f ',4)
        printwithcheck(fid,c.mdist{i}(j),'%6.2f ',7)
        printwithcheck(fid,c.Mdist{i}(j),'%6.2f ',7)
        printwithcheck(fid,c.AType{i}{j},'%s ',2)
        printwithcheck(fid,c.LocMeth{i}{j},'%s ',2)
        printwithcheck(fid,c.EvType{i}{j},'%s ',3)
        if ~isempty(c.Author{i}{j})
            fprintf(fid,'%s ',pad(c.Author{i}{j},9,'right')); 
        else
            fprintf(fid,'%s ',pad('',9,'right'));
        end
        fprintf(fid,'%s',pad(c.OrigID{i}{j},8,'left'));
        
        % spacer
        fprintf(fid,'\n');

        % check if there's any comments to write
        if ~isempty(c.Comment{i}{j})
            % check if comment contains more than one set of parentheses
            if contains(c.Comment{i}{j},')(')
                % split up each clause and print to separate lines, filling
                % in missing brackets
                str = strsplit(c.Comment{i}{j},')(');
                fprintf(fid,'%s)\n(',str{1});
                for k = 2:length(str)-1
                    fprintf(fid,'%s)\n(',str{k});
                end
                fprintf(fid,'%s\n',str{end});
            else
                % print comment if it's just a single set of brackets
                fprintf(fid,'%s\n',c.Comment{i}{j});
            end
        end
    end
    
    % spacer
    fprintf(fid,'\n');
    
    
    %% only write magnitude block for events with any magnitudes
    if any(~isnan(c.Magnitude{i}))
        
        % magnitude block header
        fprintf(fid,'Magnitude  Err Nsta Author      OrigID');
        fprintf(fid,'\n');
        
        % loop through magnitude block entries
        for j = 1:length(c.Magnitude{i})
            % write data with ISF character locations
            if ~isempty(c.MagType{i}{j})
                fprintf(fid,'%s',pad(c.MagType{i}{j},5,'right')); 
            else
                fprintf(fid,'     ');
            end
            printwithcheck(fid,c.MinMaxInd{i}{j},'%s',1)
            printwithcheck(fid,c.Magnitude{i}(j),'%4.1f ',5)
            printwithcheck(fid,c.MagErr{i}(j),'%3.1f ',4)
            printwithcheck(fid,c.MagNsta{i}(j),'%4.0f ',5)            
            if ~isempty(c.MagAuthor{i}{j})
                fprintf(fid,'%s ',pad(c.MagAuthor{i}{j},9,'right')); 
            else
                fprintf(fid,'%s ',pad('',9,'right'));
            end
            fprintf(fid,'%s',pad(c.MagOrigID{i}{j},8,'left'));
            
            % spacer
            fprintf(fid,'\n');
            
            % check if there's any comments to write
            if ~isempty(c.MagComment{i}{j})
                % check if comment contains more than one set of brackets
                if contains(c.MagComment{i}{j},')(')
                    % split up each clause and print to separate lines, 
                    % filling in missing brackets
                    str = strsplit(c.MagComment{i}{j},')(');
                    fprintf(fid,'%s)\n(',str{1});
                    for k = 2:length(str)-1
                        fprintf(fid,'%s)\n(',str{k});
                    end
                    fprintf(fid,'%s\n',str{end});
                else
                    % print comment if it's just a single set of brackets
                    fprintf(fid,'%s\n',c.MagComment{i}{j});
                end
            end
        end
        
        % spacer
        fprintf(fid,'\n');
    end
    
    
    
    %% only write phase block for events with any phases
    if ~isempty(c.PhTime{i})
        % phase block header
        fprintf(fid,['Sta     Dist  EvAz Phase        Time      TRes  ',...
            'Azim AzRes   Slow   SRes Def   SNR       Amp   Per Qual ',...
            'Magnitude    ArrID']);
        fprintf(fid,'\n');
        
        % loop through phase block entries
        for j = 1:length(c.PhTime{i})
            % write with ISF
            if ~isempty(c.Sta{i}{j})
                fprintf(fid,'%s ',pad(c.Sta{i}{j},5,'right')); 
            else
                fprintf(fid,'      ');
            end
            printwithcheck(fid,c.Dist{i}(j),'%6.2f ',7)
            printwithcheck(fid,c.EvAz{i}(j),'%5.1f ',6)
            if ~isempty(c.Phase{i}{j})
                fprintf(fid,'%s ',pad(c.Phase{i}{j},8,'right'));
            else
                fprintf(fid,'         ');
            end
            printwithcheck(fid,c.PhTime{i}{j},'%s ',13)
            printwithcheck(fid,c.TRes{i}(j),'%5.1f ',6)
            printwithcheck(fid,c.Azim{i}(j),'%5.1f ',6)
            printwithcheck(fid,c.AzRes{i}(j),'%5.1f ',6)
            printwithcheck(fid,c.Slow{i}(j),' %5.1f ',7)
            printwithcheck(fid,c.SRes{i}(j),' %5.1f ',7)
            printwithcheck(fid,c.Def{i}{j},'%s ',4)
            printwithcheck(fid,c.SNR{i}(j),'%5.1f ',6)
            printwithcheck(fid,c.Amp{i}(j),'%9.1f ',10)
            printwithcheck(fid,c.Per{i}(j),'%5.2f ',6)
            printwithcheck(fid,c.PickType{i}{j},'%s',1)
            printwithcheck(fid,c.MotionDir{i}{j},'%s',1)
            printwithcheck(fid,c.OnsetQual{i}{j},'%s ',2)
            if ~isempty(c.PhMagType{i}{j})
                fprintf(fid,'%s',pad(c.PhMagType{i}{j},5,'right')); 
            else
                fprintf(fid,'     ');
            end
            printwithcheck(fid,c.PhMagMinMax{i}{j},'%s',1)
            printwithcheck(fid,c.PhMagnitude{i}(j),'%4.1f ',5)
            fprintf(fid,'%s',pad(c.ArrID{i}{j},8,'left'));
            
            % spacer
            fprintf(fid,'\n');
            
            % check if there's any comments to write
            if ~isempty(c.PhComment{i}{j})
                % check if comment contains more than one set of brackets
                if contains(c.PhComment{i}{j},')(')
                    % split up each clause and print to separate lines, 
                    % filling in missing brackets
                    str = strsplit(c.PhComment{i}{j},')(');
                    fprintf(fid,'%s)\n(',str{1});
                    for k = 2:length(str)-1
                        fprintf(fid,'%s)\n(',str{k});
                    end
                    fprintf(fid,'%s\n',str{end});
                else
                    % print comment if it's just a single set of brackets
                    fprintf(fid,'%s\n',c.PhComment{i}{j});
                end
            end
        end
        
        % spacer
        fprintf(fid,'\n');
    end
    
    % spacer between each "unique" event
    fprintf(fid,'\n');
    
    % write status to command line
    fprintf('done.\n')
end

% close file
fclose(fid);

end

% function which allows for writing specific length string to file, which
% checks if there's no data to put in.
function printwithcheck(fid,param,form,len)

if ~isempty(param)
    if ~isnan(param)
        str = sprintf(form,param);
        if length(str)>len
            while length(str)>len && form(find(form=='.')+1)~=0
                form(find(form=='.')+1) = form(find(form=='.')+1)-1;
                str = sprintf(form,param);
            end

        elseif length(str)<len
            while length(str)<len
                form = [form,' '];
                str = sprintf(form,param);
            end
        end
        fprintf(fid,form,param);
    else
        fprintf(fid,pad('',len,'right'));
    end
else
    fprintf(fid,pad('',len,'right'));
end

end



















